
// Euler explicite version finale
function [liste_y,liste_t]=Euler(f,y0,N,T)
y=y0;liste_y=[y0];
t=0;liste_t=[0];
h=T/N;
for i=1:N
  y=y+h*f(t,y);
  t=t+h;
  liste_y=[liste_y,y];
  liste_t=[liste_t,t];
end
endfunction



// version adaptee de Newton pour la resolution de l'equation
// x - ( y + h*f(t+h,x) ) = 0
// où h,t,y sont connus fixés
// initialisation avec x = y
function [x,X] = newton4Euler(f,Df,y,h,t,Tol,MaxIter)
    n = size(y,'r');
    x = y;
    X=[];
    err = Tol+1;
    iter = 0;
    while (err>Tol)&(iter<MaxIter)
        iter = iter+1;
        DF = eye(n,n) - h*Df(t+h,x);
        F = x - (y + h*f(t+h,x));
        dx = DF\F;
        xnew = x - dx;
        err = norm(x-xnew);
        x = xnew;
        X=[X,x]
    end
endfunction


// Euler implicite par la methode de Newton
function [liste_y,liste_t]=Eulerimplicite(f,Df,y0,N,T)
    y=y0;
    liste_y=[y0];
    t=0;
    liste_t=[0];
    h=T/N;
    for i=1:N
        // resolution de l'equation y=yn+h*f(y,t_{n+1})
        y = newton4Euler(f,Df,y,h,t,1e-12,10);
        t=t+h;
        liste_y=[liste_y,y];
        liste_t=[liste_t,t];
    end
endfunction




function yp = f(t,y)
    yp = -500*(y-cos(t));
endfunction

function yp = Df(t,y)
    n = size(y,'c');
    yp = -500*eye(n,n);
endfunction

disp('-- resolution du 1er probleme raide par Euler implicite et explicite --');
disp(' donnee initiale : y(0) = 0.5 ');
disp(' intervalle de temps [0,1] ');


[liste_y2,liste_t2]=Eulerimplicite(f,Df,0.5,50,1);
figure(1);
clf;
plot(liste_t2,liste_y2);
title('solution par Euler implicite avec 50 points');


[liste_yb,liste_tb]=Euler(f,0.5,300,1);
figure(2);
clf;
plot(liste_tb,liste_yb);
title('solution par Euler explicite avec 300 points');








function yp = LV(t,y)
    a = 3;
    b = 1;
    c = 2;
    d = 2;
    yp = [a*y(1)-b*y(1)*y(2);-c*y(2)+d*y(1)*y(2)];
endfunction

function yp = DLV(t,y)
    a = 3;
    b = 1;
    c = 2;
    d = 2;
    yp = [a-b*y(2),-b*y(1);d*y(2),-c+d*y(1)];
endfunction



disp('-- resolution de Lotka-Volterra par Euler implicite et explicite --');

y0 = [1;1];
[liste_yA,liste_tA]=Euler(LV,y0,1000,10);
[liste_yB,liste_tB]=Eulerimplicite(LV,DLV,y0,1000,10);
figure(3);
clf;
plot(liste_yA(1,:),liste_yA(2,:),liste_yB(1,:),liste_yB(2,:));
title('Lotka-Volterra, solution approchee sur [0,10] - portrait de phase , 1000 points');
legend('euler explicite','euler implicite');

disp('-- resolution de Lotka-Volterra par Euler implicite et explicite --');

y0 = [1;1];
[liste_yA,liste_tA]=Euler(LV,y0,2000,10);
[liste_yB,liste_tB]=Eulerimplicite(LV,DLV,y0,2000,10);
figure(4);
clf;
plot(liste_yA(1,:),liste_yA(2,:),liste_yB(1,:),liste_yB(2,:));
title('Lotka-Volterra, solution approchee sur [0,10] - portrait de phase , 2000 points');
legend('euler explicite','euler implicite');




// programmation de Crank-Nicolson
// version adaptee de Newton pour la resolution de l'equation
// x - ( y + h/2*(f(t+h,x)+f(t,y)) ) = 0
// où h,t,y sont connus fixés
// initialisation avec x = y
function [x,X] = newton4CrankNicolson(f,Df,y,h,t,Tol,MaxIter)
    n = size(y,'r');
    x = y;
    X=[];
    err = Tol+1;
    iter = 0;
    while (err>Tol)&(iter<MaxIter)
        iter = iter+1;
        DF = eye(n,n) - h/2*Df(t+h,x);
        F = x - (y + h/2*(f(t+h,x)+f(t,y)));
        dx = DF\F;
        xnew = x - dx;
        err = norm(x-xnew);
        x = xnew;
        X=[X,x]
    end
endfunction


// Euler implicite par la methode de Newton
function [liste_y,liste_t]=CrankNicolson(f,Df,y0,N,T)
    y=y0;
    liste_y=[y0];
    t=0;
    liste_t=[0];
    h=T/N;
    for i=1:N
        // resolution de l'equation y=yn+h*f(y,t_{n+1})
        y = newton4CrankNicolson(f,Df,y,h,t,1e-12,10)
        t=t+h;
        liste_y=[liste_y,y];
        liste_t=[liste_t,t];
    end
endfunction



disp('-- resolution de Lotka-Volterra par Euler implicite et explicite, Crank-Nicolson --');

y0 = [1;1];
[liste_yA,liste_tA]=Euler(LV,y0,1000,10);
[liste_yB,liste_tB]=Eulerimplicite(LV,DLV,y0,1000,10);
[liste_yC,liste_tC]=CrankNicolson(LV,DLV,y0,1000,10);
figure(5);
clf;
plot(liste_yA(1,:),liste_yA(2,:),liste_yB(1,:),liste_yB(2,:),liste_yC(1,:),liste_yC(2,:));
title('Lotka-Volterra, approchee sur [0,10] - portrait de phase , 1000 points');
legend('Euler explicite','Euler implicite','Crank-Nicolson');


disp('la courbe solution obtenue par Crank-Nicolson semble être fermée (ce qui est une propriété vérifiée par la solution exacte : sauriez-vous le prouver ?), tandis que les courbes obtenues pour Euler explicite et implicite spiralent respectivement vers l exterieur et vers l interieur.')

